/**
 *	S3C (Simple Server-side Cache)
 *	Server-side Object Caching Framework 
 *
 *	Copyright (C) 2005  Kamran Zafar
 *
 *	This program is free software; you can redistribute it and/or
 *	modify it under the terms of the GNU General Public License
 *	as published by the Free Software Foundation; either version 2
 *	of the License, or (at your option) any later version.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License
 *	along with this program; if not, write to the Free Software
 *	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 *	Contact Info:
 *	Email: xeus.man@gmail.com
 */

package xeus.s3c;

import java.io.IOException;

import javax.naming.InitialContext;
import javax.naming.NamingException;

import org.xml.sax.SAXException;

import xeus.s3c.exception.CacheException;
import xeus.s3c.xml.parser.CacheXMLReader;

/**
 * @author Kamran Zafar
 *  
 */
public class CacheManager {

    public static Cache loadCache(String file) throws CacheException {
        CacheXMLReader reader = new CacheXMLReader();

        try {
            reader.read(file);
        } catch (SAXException e) {
            e.printStackTrace();
            throw new CacheException(
                    "Cache could not be loaded because of parsing errors in configuration file");
        } catch (IOException e) {
            e.printStackTrace();
            throw new CacheException(
                    "Cache could not be loaded because of IO errors in configuration file");
        }
        return reader.getCache();
    }

    public static Cache create(String name) throws CacheException {
        S3Cache cache = new S3Cache(name);

        try {
            InitialContext cxt = new InitialContext();
            cxt.bind("java:/" + name, cache);
        } catch (NamingException e) {
            e.printStackTrace();
            throw new CacheException(
                    "Cache could not be create: Cache with the given name may already have been loaded");
        }
        return cache;
    }

    public static Cache create(String name, int timeout, int slots)
            throws CacheException {
        S3Cache cache = new S3Cache(name, timeout, slots);
        cache.initialize();
        try {
            InitialContext cxt = new InitialContext();
            cxt.bind("java:/" + name, cache);
        } catch (NamingException e) {
            e.printStackTrace();
            throw new CacheException(
                    "Cache could not be create: Cache with the given name may already have been loaded");
        }
        return cache;
    }

    public static void unload(Cache cache) throws CacheException {
        try {
            InitialContext cxt = new InitialContext();
            cxt.unbind("java:/" + ((S3Cache) cache).getName());
        } catch (NamingException e) {
            e.printStackTrace();
            throw new CacheException("Could not unload cache");
        }
    }

    public static Cache lookup(String url) throws CacheException {
        Cache cache;
        try {
            cache = (Cache) (new InitialContext()).lookup("java:/" + url);
        } catch (NamingException e) {
            e.printStackTrace();
            throw new CacheException(
                    "Could not lookup cache: The cache may not have been loaded");
        }
        return cache;
    }
}